/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

//
// This ns-3 example demonstrates the use of a topology framework to ease 
// the construction of simulation scenarios.  
// 
// The simulation topology consists of a mixed wired and wireless
// scenario.
//
// The simulation layout consists of N backbone routers interconnected
// by an ad hoc wifi network.
// Each backbone router also has a local 802.11 network and is connected
// to a local LAN.  An additional set of (K-1) nodes are connected to
// this backbone.  Finally, a local LAN is connected to each router
// on the backbone, with L-1 additional hosts.  
//
// The nodes are populated with TCP/IP stacks, and OLSR unicast routing
// on the backbone.  An example UDP transfer is shown.  The simulator
// be configured to output tcpdumps or traces from different nodes.
//
//
//          +--------------------------------------------------------+
//          |                                                        |
//          |              802.11 ad hoc, ns-2 mobility              | 
//          |                                                        |
//          +--------------------------------------------------------+
//                   |       o o o (N backbone routers)       |
//               +--------+                               +--------+
//     wired LAN | mobile |                     wired LAN | mobile |
//    -----------| router |                    -----------| router |
//               ---------                                ---------
//                   |                                        |
//          +----------------+                       +----------------+
//          |     802.11     |                       |     802.11     |
//          |     ad-hoc     |                       |     ad-hoc     |
//          |   K-1 hosts    |                       |   K-1 hosts    |
//          +----------------+                       +----------------+
//

#include <sstream>
#include "ns3/log.h"
#include "ns3/command-line.h"
#include "ns3/ptr.h"
#include "ns3/simulator.h"
#include "ns3/nstime.h"
#include "ns3/node-list.h"
#include "ns3/olsr-helper.h"
#include "ns3/config.h"
#include "ns3/double.h"
#include "ns3/string.h"
#include "ns3/onoff-application.h"
#include "ns3/packet-sink.h"

#include "ns3/ipv4-csma-network-manager.h"
#include "ns3/ipv4-adhoc-wifi-network-manager.h"
#include "ns3/ipv4-address-allocator.h"

#include "ns3/ipv4-protocol-manager.h"
#include "ns3/ipv4.h"
#include "ns3/inet-socket-address.h"

#include "ns3/static-mobility-model.h"

#include "ns3/udp-echo-client.h"
#include "ns3/udp-echo-server.h"
#include "ns3/uinteger.h"

#include "ns3/ascii-trace.h"
#include "ns3/pcap-trace.h"

using namespace ns3;

NS_LOG_COMPONENT_DEFINE ("MixedWirelessTopology");

Ipv4Address g_routerAddr;

class MyIpv4ProtocolManager : public Ipv4ProtocolManager
{
public:
  void CreateProtocols (Ptr<Node> node, bool external);
};

  void
MyIpv4ProtocolManager::CreateProtocols (Ptr<Node> node, bool external)
{
  NS_LOG_FUNCTION;
//
// Create the usual protocols.
//
  Ipv4ProtocolManager::CreateProtocols (node, external);
//
// External nodes are nodes that are built elsewhere and added to the network
// manager.  In our case the single external node is the backbone node.  We 
// want to set a default route to there on all of the other nodes.  
//
  if (!external)
    {
      Ptr<Ipv4> ipv4 = node->GetObject<Ipv4> ();
      ipv4->SetDefaultRoute (g_routerAddr, uint32_t (1));
    }
}

  int 
main (int argc, char *argv[])
{
//
// First, we declare and initialize a few local variables that control some 
// simulation parameters.
//
  uint32_t backboneNodes = 10;
  uint32_t leafNodes = 10;
  uint32_t lanNodes = 5;
  uint32_t stopTime = 10;

//
// For convenience, we add the local variables to the command line argument
// system so that they can be overridden with flags such as 
// "--backboneNodes=20"
//
  CommandLine::AddArgValue("backboneNodes","number of backbone nodes", 
    backboneNodes);

  CommandLine::AddArgValue("leafNodes","number of leaf nodes", 
    leafNodes);

  CommandLine::AddArgValue("lanNodes","number of LAN nodes", 
    lanNodes);

  CommandLine::AddArgValue("stopTime","simulation stop time (seconds)", 
    stopTime);

//
// Configuration of the ns-3 system (e.g.,scheduler type) is done by setting
// global variables as follows.
//
  Config::SetDefault ("/NodeList/*/DeviceList/*/Phy/TxGain", Double (2.0));

  Config::SetDefault ("OnOffApplication::PacketSize", String ("210"));
  Config::SetDefault ("OnOffApplication::DataRate", String ("448kb/s"));

//
// THe system global variables and the local values added to the argument
// system can be overridden by command line arguments by using this call.
//
  CommandLine::Parse (argc, argv);

  Ptr<Ipv4AddressAllocator> ipAddrs = CreateObject<Ipv4AddressAllocator> ();
  ipAddrs->Init ("192.168.0.0", "255.255.255.0", "0.0.0.1");

  Ptr<Ipv4AdhocWifiNetworkManager> backboneNet = 
    CreateObject<Ipv4AdhocWifiNetworkManager> (ipAddrs);
  backboneNet->Create (backboneNodes);
  backboneNet->Configure ();

//
// The ad-hoc backbone network nodes need a mobility model.
// n.b. would use a MobilityManager here (not yet implemented)
//
  for (uint32_t i = 0; i < backboneNet->GetNodeManager ()->GetNNodes (); ++i)
    {
      Ptr<MobilityModel> mobility = CreateObject<StaticMobilityModel> ();
      mobility->SetPosition (Vector (i, 0., 0.));
      Ptr<Node> node = backboneNet->GetNodeManager ()->GetNode (i);
      node->AggregateObject (mobility);
    }

//
// Create the LAN networks -- one LAN per backbone node.
//
  std::vector<Ptr<Ipv4CsmaNetworkManager> > wiredNets (backboneNodes);
  for (uint32_t i = 0; i < backboneNodes; ++i)
    {
      ipAddrs->AllocateNetwork ();
      g_routerAddr = ipAddrs->GetAddress ();
      Ptr<Ipv4CsmaNetworkManager> wiredNet = 
        CreateObject<Ipv4CsmaNetworkManager> (ipAddrs);
      Ptr<MyIpv4ProtocolManager> pm = CreateObject<MyIpv4ProtocolManager> ();
      wiredNet->SetProtocolManager (pm);
      wiredNet->AddNode (backboneNet->GetNodeManager ()->GetNode (i));
      wiredNet->Create (lanNodes - 1);
      wiredNet->Configure ();
      wiredNets[i] = wiredNet;
    }

//
// Now, create the wireless "leaf" networks
//
  std::vector<Ptr<Ipv4AdhocWifiNetworkManager> > leafNets (backboneNodes);
  for (uint32_t i = 0; i < backboneNodes; ++i)
    {
      ipAddrs->AllocateNetwork ();
      g_routerAddr = ipAddrs->GetAddress ();
      Ptr<Ipv4AdhocWifiNetworkManager> leafNet = 
        CreateObject<Ipv4AdhocWifiNetworkManager> (ipAddrs);
      Ptr<MyIpv4ProtocolManager> pm = CreateObject<MyIpv4ProtocolManager> ();
      leafNet->SetProtocolManager (pm);
      leafNet->AddNode (backboneNet->GetNodeManager ()->GetNode (i));
      leafNet->Create (leafNodes - 1);
      leafNet->Configure ();
//
// The nodes in the wireless leaf network are also mobile,
//
      for (uint32_t i = 0; i < leafNet->GetNodeManager ()->GetNNodes (); ++i)
        {
          Ptr<MobilityModel> mobility = CreateObject<StaticMobilityModel> ();
          mobility->SetPosition (Vector (0., i, 0.));
          Ptr<Node> leafNode = leafNet->GetNodeManager ()->GetNode (i);
          leafNode->AggregateObject (mobility);
        }
    }

//
// At this point, we have built the various networks described at the top of 
// the file, connected them, added Ipv4 protocol stacks and interfaces
// to each node and assigned Ipv4 addresses to each interface.
// n.b. would use a RoutingManager here (not yet implemented)
//
  OlsrHelper olsr;

//
// Enable OLSR routing for the backbone network.
//
  for (uint32_t i = 0; i < backboneNodes; ++i)
    {
      Ptr<Node> n = backboneNet->GetNodeManager ()->GetNode (i);
      NS_LOG_LOGIC ("Installing OLSR for node " << n->GetId ());
      olsr.Enable (n);
    }

// Create the OnOff application to send UDP datagrams of size
// 210 bytes at a rate of 448 Kb/s
  NS_LOG_INFO ("Create Applications.");
  uint16_t port = 9;   // Discard port (RFC 863)

  Ptr<Node> appSource = NodeList::GetNode (11);
  Ptr<Node> appSink = NodeList::GetNode (13);

  Ptr<OnOffApplication> ooff = 
    CreateObjectWith<OnOffApplication> (
           "Node", appSource,
           "Remote", Address (InetSocketAddress ("192.168.1.5", port)),
           "Protocol", TypeId::LookupByName ("Udp"),                           
           "OnTime", ConstantVariable(1),
           "OffTime", ConstantVariable(0));
  appSource->AddApplication (ooff);            
// Start the application
  ooff->Start (Seconds(3.0));
  ooff->Stop (Seconds(20.0));
  
// Create an optional packet sink to receive these packets
  Ptr<PacketSink> sink = 
    CreateObjectWith<PacketSink> (
           "Node", appSink,
           "Local", Address (InetSocketAddress (Ipv4Address::GetAny (), port)), 
           "Protocol", TypeId::LookupByName ("Udp"));
  appSink->AddApplication (sink);      
// Start the sink
  sink->Start (Seconds (5.0));
  sink->Stop (Seconds(10.0));

//
// Let's trace the queue operations of the devices on the CSMA network 
// connected to the last backbone node (all nodes but the backbone node).
//
  AsciiTrace asciiTrace ("mixed-wireless-topology.tr");
  {
//
// Build a path string for the nodes we want to trace.
//
    Ptr<NodeManager> nm = wiredNets[backboneNodes - 1]->GetNodeManager ();
    uint32_t startingId = nm->GetNode (1)->GetId ();
    uint32_t endingId = nm->GetNode (nm->GetNNodes() - 1)->GetId ();
    std::ostringstream path;
    path << "/NodeList/[" << startingId << "-" << endingId << "]/DeviceList/*";
//
// Enable the traces.
//
    asciiTrace.TraceQueues (path.str ());
    asciiTrace.TraceNetDeviceRx (path.str ());
  }
//
// Let's do a pcap trace on the backbone devices
//
  PcapTrace pcaptrace ("mixed-wireless-topology.pcap");
  pcaptrace.TraceIp ("/NodeList/[0-5]");

  Simulator::StopAt (Seconds (stopTime));
  Simulator::Run ();    
  Simulator::Destroy ();
}
