/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

//
// This ns-3 example demonstrates the use of helper functions to ease 
// the construction of simulation scenarios.  
// 
// The simulation topology consists of a mixed wired and wireless
// scenario in which a hierarchical mobility model is used.
//
// The simulation layout consists of N backbone routers interconnected
// by an ad hoc wifi network.
// Each backbone router also has a local 802.11 network and is connected
// to a local LAN.  An additional set of (K-1) nodes are connected to
// this backbone.  Finally, a local LAN is connected to each router
// on the backbone, with L-1 additional hosts.  
//
// The nodes are populated with TCP/IP stacks, and OLSR unicast routing
// on the backbone.  An example UDP transfer is shown.  The simulator
// be configured to output tcpdumps or traces from different nodes.
//
//
//          +--------------------------------------------------------+
//          |                                                        |
//          |              802.11 ad hoc, ns-2 mobility              | 
//          |                                                        |
//          +--------------------------------------------------------+
//                   |       o o o (N backbone routers)       |
//               +--------+                               +--------+
//     wired LAN | mobile |                     wired LAN | mobile |
//    -----------| router |                    -----------| router |
//               ---------                                ---------
//                   |                                        |
//          +----------------+                       +----------------+
//          |     802.11     |                       |     802.11     |
//          |      net       |                       |       net      |
//          |   K-1 hosts    |                       |   K-1 hosts    |
//          +----------------+                       +----------------+
//

#include "ns3/log.h"
#include "ns3/command-line.h"
#include "ns3/ptr.h"
#include "ns3/simulator.h"
#include "ns3/nstime.h"
#include "ns3/config.h"
#include "ns3/double.h"
#include "ns3/string.h"

#include "ns3/node-list.h"

#include "ns3/helper.h"
#include "ns3/csma-netdevice-helper.h"
#include "ns3/infra-wifi-netdevice-helper.h"
#include "ns3/adhoc-wifi-netdevice-helper.h"
#include "ns3/ipv4-protocol-helper.h"
#include "ns3/ipv4-address-helper.h"
#include "ns3/ipv4-address-maker.h"
#include "ns3/inet-socket-address.h"
#include "ns3/onoff-application.h"
#include "ns3/packet-sink.h"

#include "ns3/mobility-model.h"
#include "ns3/mobility-helper.h"
#include "ns3/position-allocator.h"
#include "ns3/rectangle.h"

#include "ns3/udp-echo-client.h"
#include "ns3/udp-echo-server.h"
#include "ns3/uinteger.h"

#include "ns3/olsr-helper.h"

#include "ns3/ascii-trace.h"
#include "ns3/pcap-trace.h"

using namespace ns3;

//
// Define logging keyword for this file
//
NS_LOG_COMPONENT_DEFINE ("MixedWireless");

//
// This function will be used below as a trace sink
// 
#ifdef ENABLE_FOR_TRACING_EXAMPLE
static void
CourseChangeCallback (std::string path, Ptr<const MobilityModel> model)
{
  Vector position = model->GetPosition ();
  std::cout << "CourseChange " << path << " x=" << position.x << ", y=" << position.y << ", z=" << position.z << std::endl;
}
#endif

int 
main (int argc, char *argv[])
{
  //
  // First, we declare and initialize a few local variables that control some 
  // simulation parameters.
  //
  uint32_t backboneNodes = 10;
  uint32_t infraNodes = 10;
  uint32_t lanNodes = 5;
  uint32_t stopTime = 60;

  // An IP address generator is used in the topology construction 
  Ipv4AddressMaker ipAddrs;
  ipAddrs.Init ("192.168.0.0", "255.255.255.0", "0.0.0.1");

  //
  // For convenience, we add the local variables to the command line argument
  // system so that they can be overridden with flags such as 
  // "--backboneNodes=20"
  //
  CommandLine::AddArgValue("backboneNodes","number of backbone nodes", 
    backboneNodes);
  CommandLine::AddArgValue("infraNodes","number of leaf nodes", 
    infraNodes);
  CommandLine::AddArgValue("lanNodes","number of LAN nodes", 
    lanNodes);
  CommandLine::AddArgValue("stopTime","simulation stop time (seconds)", 
    stopTime);

  //
  // Simulation defaults can be set here.  Two ways of doing this
  // are illustrated (one with a config path, and the other by explicit
  // referencing of the TypeId)
  //
  Config::SetDefault ("/NodeList/*/DeviceList/*/Phy/TxGain", Double (2.0));
  Config::SetDefault ("OnOffApplication::PacketSize", String ("210"));
  Config::SetDefault ("OnOffApplication::DataRate", String ("448kb/s"));

  //
  // The system global variables and the local values added to the argument
  // system can be overridden by command line arguments by using this call.
  //
  CommandLine::Parse (argc, argv);

  /////////////////////////////////////////////////////////////////////////// 
  //                                                                       //
  // Construct the backbone                                                //
  //                                                                       //
  /////////////////////////////////////////////////////////////////////////// 

  //
  // Create a container to manage the nodes of the adhoc (backbone) network.
  // Then create the rest of the nodes we'll need.
  //
  NodeContainer backbone;
  backbone.Create (backboneNodes);
  //
  // Create the backbone wifi net devices and install them into the nodes in 
  // our container
  //
  AdhocWifiNetDeviceHelper backboneDevices;
  backboneDevices.Create (backboneNodes);
  backboneDevices.Install (backbone);
  //
  // Add the IPv4 protocol stack to the nodes in our container
  //
  Ipv4ProtocolHelper protocols;
  protocols.Install (backbone);
  //
  // Assign IPv4 addresses to the device drivers (actually to the associated
  // IPv4 interfaces) we just created.
  //
  Ipv4AddressHelper addresses (ipAddrs);
  addresses.Install (backboneDevices);

  //
  // The ad-hoc network nodes need a mobility model so we aggregate one to 
  // each of the nodes we just finished building.  
  //
  Ptr<ListPositionAllocator> initialAlloc = 
    CreateObject<ListPositionAllocator> ();
  for (uint32_t i = 0; i < backbone.GetN (); ++i)
    {
      initialAlloc->Add (Vector (i, 0., 0.));
    }
   MobilityHelper mobility; 
   mobility.EnableNotifier ();
   mobility.SetPositionAllocator (initialAlloc);
   mobility.SetMobilityModel ("RandomDirection2dMobilityModel",
                              "Bounds", Rectangle (0, 1000, 0, 1000),
                              "Speed", ConstantVariable (2000),
                              "Pause", ConstantVariable (0.2));
   mobility.Layout (backbone.Begin (), backbone.End ());

  /////////////////////////////////////////////////////////////////////////// 
  //                                                                       //
  // Construct the LANs                                                    //
  //                                                                       //
  /////////////////////////////////////////////////////////////////////////// 

  for (uint32_t i = 0; i < backboneNodes; ++i)
    {
      NS_LOG_INFO ("Configuring local area network for backbone node " << i);
      //
      // Allocate a new network prefix for each LAN, according to the
      // network mask initialized above
      //
      ipAddrs.AllocateNetwork ();
      //
      // Create a container to manage the nodes of the LAN.  Pick one of 
      // the backbone nodes to be part of the LAN and first add it to 
      // the container.  Then create the rest of the nodes we'll need.
      //
      NodeContainer lan;
      lan.Add (backbone.Get (i));
      lan.Create (lanNodes - 1);
      //
      // Create the CSMA net devices and install them into the nodes in our 
      // collection.
      //
      CsmaNetDeviceHelper lanDevs;
      lanDevs.Create (lanNodes);
      lanDevs.Install (lan);
      //
      // Add the IPv4 protocol stack to the nodes in our container
      //
      Ipv4ProtocolHelper protocols;
      protocols.Install (lan);
      //
      // Assign IPv4 addresses to the device drivers (actually to the 
      // associated IPv4 interfaces) we just created.
      //
      Ipv4AddressHelper addresses (ipAddrs); 
      addresses.Install (lanDevs);
    }

  /////////////////////////////////////////////////////////////////////////// 
  //                                                                       //
  // Construct the mobile networks                                         //
  //                                                                       //
  /////////////////////////////////////////////////////////////////////////// 

  for (uint32_t i = 0; i < backboneNodes; ++i)
    {
      NS_LOG_INFO ("Configuring wireless network for backbone node " << i);
      //
      // Allocate a new network prefix for each mobile network, according to 
      // the network mask initialized above
      //
      ipAddrs.AllocateNetwork ();
      //
      // Create a container to manage the nodes of the network.  Pick one of 
      // the backbone nodes to be part of the network and first add it to 
      // the container.  Then create the rest of the nodes we'll need.
      //
      NodeContainer infra;
      infra.Add (backbone.Get (i));
      infra.Create (infraNodes - 1);
      //
      // Create another ad hoc network and devices
      //
      AdhocWifiNetDeviceHelper infraDevs;
      infraDevs.Create (infraNodes);
      infraDevs.Install (infra);
      //
      // Add the IPv4 protocol stack to the nodes in our container
      //
      Ipv4ProtocolHelper protocols;
      protocols.Install (infra);
      //
      // Assign IPv4 addresses to the device drivers (actually to the associated
      // IPv4 interfaces) we just created.
      //
      Ipv4AddressHelper addresses (ipAddrs);
      addresses.Install (infraDevs);
      //
      // The new wierless nodes need a mobility model so we aggregate one 
      // to each of the nodes we just finished building.
      //
      Ptr<ListPositionAllocator> subnetAlloc = 
        CreateObject<ListPositionAllocator> ();
      for (uint32_t j = 0; j < infra.GetN (); ++j)
        {
          subnetAlloc->Add (Vector (0.0, j, 0.0));
        }
      mobility.EnableNotifier ();
      mobility.PushReferenceMobilityModel (backbone.Get (i));
      mobility.SetPositionAllocator (subnetAlloc);
      mobility.SetMobilityModel ("RandomDirection2dMobilityModel",
                                 "Bounds", Rectangle (-25, 25, -25, 25),
                                 "Speed", ConstantVariable (30),
                                 "Pause", ConstantVariable (0.4));
      mobility.Layout (infra.Begin (), infra.End ());
    }

  /////////////////////////////////////////////////////////////////////////// 
  //                                                                       //
  // Routing configuration                                                 //
  //                                                                       //
  /////////////////////////////////////////////////////////////////////////// 

  OlsrHelper olsr;
  for (uint32_t i = 0; i < backboneNodes; ++i)
    {
      Ptr<Node> n = backbone.Get (i);
      NS_LOG_INFO ("Installing OLSR for node " << n->GetId ());
      olsr.Enable (n);
    }

  /////////////////////////////////////////////////////////////////////////// 
  //                                                                       //
  // Application configuration                                             //
  //                                                                       //
  /////////////////////////////////////////////////////////////////////////// 

  // Create the OnOff application to send UDP datagrams of size
  // 210 bytes at a rate of 448 Kb/s, between two nodes

  NS_LOG_INFO ("Create Applications.");
  uint16_t port = 9;   // Discard port (RFC 863)
  Ptr<Node> appSource = NodeList::GetNode (11);  
  Ptr<Node> appSink = NodeList::GetNode (13);  

  Ptr<OnOffApplication> ooff = 
    CreateObjectWith<OnOffApplication> (
      "Node", appSource,                                         
      "Remote", Address (InetSocketAddress ("192.168.1.5", port)),             
      "Protocol", TypeId::LookupByName ("Udp"),                            
      "OnTime", ConstantVariable (1), 
      "OffTime", ConstantVariable (0));
  appSource->AddApplication (ooff);
  // Start the application
  ooff->Start (Seconds(3.0));
  ooff->Stop (Seconds(20.0));
  
  // Create an optional packet sink to receive these packets
  Ptr<PacketSink> sink = 
    CreateObjectWith<PacketSink> (
      "Node", appSink,                                  
      "Local", Address (InetSocketAddress (Ipv4Address::GetAny (), port)),     
      "Protocol", TypeId::LookupByName ("Udp"));
  appSink->AddApplication (sink);
  // Start the sink
  sink->Start (Seconds (3.0));
 
  /////////////////////////////////////////////////////////////////////////// 
  //                                                                       //
  // Tracing configuration                                                 //
  //                                                                       //
  /////////////////////////////////////////////////////////////////////////// 

  NS_LOG_INFO ("Configure Tracing.");
  //
  // Let's set up some ns-2-like ascii traces, using another helper class
  //
  AsciiTrace asciiTrace ("mixed-wireless-topology.tr");
  // 
  // Look at nodes 11, 13 only
  // 
  asciiTrace.TraceQueues ("/NodeList/11|13/DeviceList/0");
  asciiTrace.TraceNetDeviceRx ("/NodeList/11|13/DeviceList/0");
  //
  // Let's do a pcap trace on the backbone devices
  //
  PcapTrace pcaptrace ("mixed-wireless.pcap");
  pcaptrace.TraceIp ("/NodeList/[0-5]");

#ifdef ENABLE_FOR_TRACING_EXAMPLE
  Config::ConnectWithContext ("/NodeList/*/$MobilityModelNotifier/CourseChange",
    MakeCallback (&CourseChangeCallback));
#endif

  /////////////////////////////////////////////////////////////////////////// 
  //                                                                       //
  // Run simulation                                                        //
  //                                                                       //
  /////////////////////////////////////////////////////////////////////////// 

  NS_LOG_INFO ("Run Simulation.");
  Simulator::StopAt (Seconds (stopTime));
  Simulator::Run ();    
  Simulator::Destroy ();
}
