/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ns3/assert.h"
#include "helper.h"

NS_LOG_COMPONENT_DEFINE ("Helper");

namespace ns3 {

NodeContainer_::NodeContainer_ ()
{
  NS_LOG_FUNCTION;
}

NodeContainer_::~NodeContainer_ ()
{
  NS_LOG_FUNCTION;
}

  void
NodeContainer_::PreCreate (void)
{
  NS_LOG_FUNCTION;
}

  void
NodeContainer_::Create (uint32_t n)
{
  NS_LOG_FUNCTION;
  PreCreate ();
  for (uint32_t i = 0; i < n; ++i)
    {
      Ptr<Node> node = CreateNode ();
      m_nodes.push_back (std::make_pair (node, false));
    }
  PostCreate ();
}

  void
NodeContainer_::PostCreate (void)
{
  NS_LOG_FUNCTION;
}

  void
NodeContainer_::Add (Ptr<Node> node)
{
  NS_LOG_FUNCTION;
  m_nodes.push_back (std::make_pair (node, true));
}

  bool
NodeContainer_::NodeIsExternal (Ptr<Node> node) const
{
  NS_LOG_FUNCTION;
  for (uint32_t i = 0; i < GetNNodes (); ++i)
    {
      if (node == m_nodes[i].first)
        {
          return m_nodes[i].second;
        }
    }
  NS_ASSERT_MSG (0, "NodeContainer_::NodeIsExternal(): Unknown node");
  return false;
}

  Ptr<Node>
NodeContainer_::CreateNode (void)
{
  NS_LOG_FUNCTION;
  return CreateObject<Node> ();
}

  uint32_t
NodeContainer_::GetNNodes (void) const
{
  NS_LOG_FUNCTION;
  return m_nodes.size ();
}

  Ptr<Node> 
NodeContainer_::GetNode (uint32_t index) const
{
  NS_LOG_FUNCTION;
  return m_nodes[index].first;
}

NetDeviceHelper::NetDeviceHelper ()
  : m_channel (0)
{
  NS_LOG_FUNCTION;
}

NetDeviceHelper::~NetDeviceHelper ()
{
  NS_LOG_FUNCTION;
}

  void
NetDeviceHelper::Create (uint32_t n)
{
  NS_LOG_FUNCTION;
  PreCreate ();
  m_channel = CreateChannel ();

  for (uint32_t i = 0; i < n; ++i)
    {
      Ptr<NetDevice> device = CreateNetDevice ();
      m_devices.push_back (device);
    }
  PostCreate ();
}

  void
NetDeviceHelper::PreCreate (void)
{
  NS_LOG_FUNCTION;
}

  void
NetDeviceHelper::PostCreate (void)
{
  NS_LOG_FUNCTION;
}

  Ptr<NetDevice>
NetDeviceHelper::CreateNetDevice (void)
{
  NS_LOG_FUNCTION;
  return 0;
}

  Ptr<Channel>
NetDeviceHelper::CreateChannel (void)
{
  NS_LOG_FUNCTION;
  return 0;
}

  void
NetDeviceHelper::AttachNetDevice (Ptr<NetDevice> device)
{
  NS_LOG_FUNCTION;
}

  void
NetDeviceHelper::Install (NodeContainer &nodes)
{
  NS_LOG_FUNCTION;
  PreInstall ();
  uint32_t nNodes = nodes.GetN ();
  uint32_t nDevices = GetNDevices();

  NS_ASSERT_MSG (nNodes >= nDevices, "NetDeviceHelper::Install(): "
    "More NetDevices than Nodes");
  for (uint32_t i = 0; i < nNodes; ++i)
    {
      Ptr<Node> node = nodes.Get (i);
      Ptr<NetDevice> device = GetDevice (i);
      NS_LOG_LOGIC ("Adding device " << device << " to node " << node);
      device->SetNode (node);
      node->AddDevice (device);
    }
  PostInstall ();
}

  void
NetDeviceHelper::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
NetDeviceHelper::PostInstall (void)
{
  NS_LOG_FUNCTION;
}

  uint32_t 
NetDeviceHelper::GetNDevices (void) const
{
  NS_LOG_FUNCTION;
  return m_devices.size ();
}

  Ptr<NetDevice> 
NetDeviceHelper::GetDevice (uint32_t index) const
{
  NS_LOG_FUNCTION;
  return m_devices[index];
}

  Ptr<Channel> 
NetDeviceHelper::GetChannel (void) const
{
  NS_LOG_FUNCTION;
  return m_channel;
}

ProtocolHelper::ProtocolHelper ()
{
  NS_LOG_FUNCTION;
}

ProtocolHelper::~ProtocolHelper ()
{
  NS_LOG_FUNCTION;
}

  void
ProtocolHelper::Install (NodeContainer &nodes)
{
  NS_LOG_FUNCTION;
  PreInstall ();
  uint32_t nNodes = nodes.GetN ();
  for (uint32_t i = 0; i < nNodes; ++i)
    {
      Ptr<Node> node = nodes.Get (i);
      CreateProtocols (node);
    }
  PostInstall ();
}

  void
ProtocolHelper::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
ProtocolHelper::PostInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
ProtocolHelper::CreateProtocols (Ptr<Node> node)
{
  NS_LOG_FUNCTION;
}

AddressHelper::AddressHelper ()
{
  NS_LOG_FUNCTION;
}

AddressHelper::~AddressHelper ()
{
  NS_LOG_FUNCTION;
}

  void
AddressHelper::Install (NetDeviceHelper &devices)
{
  NS_LOG_FUNCTION;
  PreInstall ();
  uint32_t nDevices = devices.GetNDevices();
  for (uint32_t i = 0; i < nDevices; ++i)
    {
      Ptr<NetDevice> device = devices.GetDevice (i);
      NS_ASSERT_MSG(device, "AddressHelper::Install(): bad device");
      AssignAddress (device);
    }
  PostInstall ();
}

  void
AddressHelper::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
AddressHelper::PostInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
AddressHelper::AssignAddress (Ptr<NetDevice> device)
{
  NS_LOG_FUNCTION;
}

ApplicationHelper::ApplicationHelper ()
{
  NS_LOG_FUNCTION;
}

ApplicationHelper::~ApplicationHelper ()
{
  NS_LOG_FUNCTION;
}

  void
ApplicationHelper::Install (NodeContainer &nodes)
{
  NS_LOG_FUNCTION;
  PreInstall ();
  uint32_t nNodes = nodes.GetN ();
  for (uint32_t i = 0; i < nNodes; ++i)
    {
      Ptr<Node> node = nodes.Get (i);
      NS_ASSERT_MSG(node, "ApplicationHelper::Install(): bad node");
      CreateApplications (node);
    }
  PostInstall ();
}

  void
ApplicationHelper::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
ApplicationHelper::PostInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
ApplicationHelper::CreateApplications (Ptr<Node> node)
{
  NS_LOG_FUNCTION;
}

} // namespace ns3
