/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/net-device.h"
#include "internet-protocol-stack.h"
#include "ipv4-l4-demux.h"
#include "udp-l4-protocol.h"
#include "tcp-l4-protocol.h"
#include "ipv4-l3-protocol.h"
#include "arp-l3-protocol.h"
#include "udp-impl.h"
#include "tcp-impl.h"
#include "ipv4-impl.h"

namespace ns3 {

/**
 * \ingroup protocolStacks
 *
 * \section InternetStack Overview
 *
 * The InternetStack module contains an implementation of TCP, UDP, and
 * IPv4.  Ns-3 Applications sit above this module, and ns-3 NetDevices
 * below.  
 *
 * The model here is conceptually similar to getting a CD from a software 
 * vendor that includes all of the constituent parts required to install a
 * protocol stack.  You would normally insert this CD into your computer and
 * install the protocol stack.  We provide a function to do this for every
 * common protocol stack.
 *
 * In this case, the function to install the Internet Protocol Stack is,
 * perhaps not too surprisingly called, InstallInternetProtocolStack.
 */

/*
 * \brief Install the Internet Protocol Stack onto a node.
 *
 * This method assembles the required layer-3/4 stack oieces required to
 * implement an Internet Protocol Stack from its constituent parts.  These
 * parts include implementations of TCP, UDP, IPv4, and ARP.
 */
  void
InstallInternetProtocolStack (Ptr<Node> node)
{
  Ptr<Ipv4L3Protocol> ipv4 = CreateObjectWith<Ipv4L3Protocol> (
    "Node", node);
  Ptr<ArpL3Protocol> arp = CreateObjectWith<ArpL3Protocol> (
    "Node", node);

  node->RegisterProtocolHandler (MakeCallback (&Ipv4L3Protocol::Receive, 
    PeekPointer (ipv4)), Ipv4L3Protocol::PROT_NUMBER, 0);
  node->RegisterProtocolHandler (MakeCallback (&ArpL3Protocol::Receive, 
    PeekPointer (arp)), ArpL3Protocol::PROT_NUMBER, 0);

  Ptr<Ipv4L4Demux> ipv4L4Demux = CreateObjectWith<Ipv4L4Demux> (
    "Node", node);
  Ptr<UdpL4Protocol> udp = CreateObjectWith<UdpL4Protocol> (
    "Node", node);
  Ptr<TcpL4Protocol> tcp = CreateObjectWith<TcpL4Protocol> (
    "Node", node);

  ipv4L4Demux->Insert (udp);
  ipv4L4Demux->Insert (tcp);

  Ptr<UdpImpl> udpImpl = CreateObject<UdpImpl> (udp);
  Ptr<TcpImpl> tcpImpl = CreateObject<TcpImpl> (tcp);
  Ptr<Ipv4Impl> ipv4Impl = CreateObject<Ipv4Impl> (ipv4);

  node->AggregateObject (ipv4);
  node->AggregateObject (arp);
  node->AggregateObject (ipv4Impl);
  node->AggregateObject (udpImpl);
  node->AggregateObject (tcpImpl);
  node->AggregateObject (ipv4L4Demux);
}

} //namespace ns3
