/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ns3/attribute.h"
#include "ns3/boolean.h"
#include "ns3/wifi-channel.h"
#include "ns3/wifi-net-device.h"
#include "ns3/wifi-mac.h"
#include "ns3/wifi-phy.h"
#include "ns3/wifi-remote-station-manager.h"
#include "ns3/propagation-delay-model.h"
#include "ns3/propagation-loss-model.h"
#include "ns3/nstime.h"

#include "infra-wifi-netdevice-manager.h"

NS_LOG_COMPONENT_DEFINE ("InfraWifiNetDeviceManager");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (InfraWifiNetDeviceManager);

  TypeId 
InfraWifiNetDeviceManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("InfraWifiNetDeviceManager")
    .SetParent<NetDeviceManager> ();

  return tid;
}

InfraWifiNetDeviceManager::InfraWifiNetDeviceManager (
  std::string ssid,
  uint32_t apIndex)
: 
  m_ssidString (ssid), 
  m_apIndex (apIndex)
{
  NS_LOG_FUNCTION;
}

InfraWifiNetDeviceManager::~InfraWifiNetDeviceManager ()
{
  NS_LOG_FUNCTION;
}

  Ptr<Channel>
InfraWifiNetDeviceManager::CreateChannel (void)
{
  NS_LOG_FUNCTION;
  Ptr<WifiChannel> channel = CreateObjectWith<WifiChannel> ();

  channel->SetPropagationDelayModel (
    CreateObjectWith<ConstantSpeedPropagationDelayModel> ());

  Ptr<LogDistancePropagationLossModel> log = 
    CreateObjectWith<LogDistancePropagationLossModel> ();
  log->SetReferenceModel (CreateObjectWith<FriisPropagationLossModel> ());
  channel->SetPropagationLossModel (log);

  return channel;
}

  void
InfraWifiNetDeviceManager::Create (uint32_t n)
{
  NS_LOG_FUNCTION;
  PreCreate ();
  m_channel = CreateChannel ();

  for (uint32_t i = 0; i < n; ++i)
    {
      Ptr<NetDevice> device;
      if (i == m_apIndex) 
        {
          device = CreateApNetDevice ();
        }
      else
        {
          device = CreateStaNetDevice ();
        }
      m_devices.push_back (device);
    }
  PostCreate ();
}

  Ptr<NetDevice>
InfraWifiNetDeviceManager::CreateNetDevice (void)
{
  NS_LOG_FUNCTION;
  NS_ASSERT_MSG (0,"InfraWifiNetDeviceManager::CreateNetDevice(): "
    "Illegal in this context -- Two kinds of net devices");
  return 0;
}

  Ptr<NetDevice>
InfraWifiNetDeviceManager::CreateStaNetDevice (void)
{
  NS_LOG_FUNCTION;
  Ssid ssid = Ssid ("wifi-default");

  m_stationManager.SetTypeId ("ArfWifiManager");
  m_phy.SetTypeId ("WifiPhy");
  m_mac.SetTypeId ("NqstaWifiMac");
  m_mac.Set ("Ssid", ssid);
  m_mac.Set ("ActiveProbing", Boolean (false));

  Ptr<WifiNetDevice> device = CreateObject<WifiNetDevice> ();
  Ptr<WifiRemoteStationManager> manager = 
    m_stationManager.Create<WifiRemoteStationManager> ();
  Ptr<WifiMac> mac = m_mac.Create<WifiMac> ();
  Ptr<WifiPhy> phy = m_phy.Create<WifiPhy> ();
  device->Setup (mac, phy, manager, m_channel->GetObject<WifiChannel> ());
  return device;
}

  Ptr<NetDevice>
InfraWifiNetDeviceManager::CreateApNetDevice (void)
{
  NS_LOG_FUNCTION;
  Ssid ssid = Ssid ("wifi-default");

  m_stationManager.SetTypeId ("ArfWifiManager");
  m_phy.SetTypeId ("WifiPhy");
  m_mac.SetTypeId ("NqapWifiMac");
  m_mac.Set ("Ssid", ssid);
  m_mac.Set ("BeaconGeneration", Boolean (true));
  m_mac.Set ("BeaconInterval", Seconds (2.5));

  Ptr<WifiNetDevice> device = CreateObject<WifiNetDevice> ();
  Ptr<WifiRemoteStationManager> manager = 
    m_stationManager.Create<WifiRemoteStationManager> ();
  Ptr<WifiMac> mac = m_mac.Create<WifiMac> ();
  Ptr<WifiPhy> phy = m_phy.Create<WifiPhy> ();
  device->Setup (mac, phy, manager, m_channel->GetObject<WifiChannel> ());
  return device;
}

  void
InfraWifiNetDeviceManager::AttachDevice (Ptr<NetDevice> device)
{
  NS_LOG_FUNCTION;
  NS_ASSERT_MSG (0, "AttachDevice done as a side-effect of CreateNetDevice");
}

} // namespace ns3
