/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "topology.h"
#include "ns3/assert.h"
#include "ns3/log.h"
#include "ns3/internet-protocol-stack.h"

NS_LOG_COMPONENT_DEFINE ("Topology");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (NodeManager);

  TypeId 
NodeManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("NodeManager")
    .SetParent<Object> ()
    .AddConstructor<NodeManager> ();
  return tid;
}

NodeManager::NodeManager ()
{
  NS_LOG_FUNCTION;
}

NodeManager::~NodeManager ()
{
  NS_LOG_FUNCTION;
}

  void
NodeManager::PreCreate (void)
{
  NS_LOG_FUNCTION;
}

  void
NodeManager::Create (uint32_t n)
{
  NS_LOG_FUNCTION;
  PreCreate ();
  for (uint32_t i = 0; i < n; ++i)
    {
      Ptr<Node> node = CreateNode ();
      m_nodes.push_back (std::make_pair (node, false));
    }
  PostCreate ();
}

  void
NodeManager::PostCreate (void)
{
  NS_LOG_FUNCTION;
}

  void
NodeManager::Add (Ptr<Node> node)
{
  NS_LOG_FUNCTION;
  m_nodes.push_back (std::make_pair (node, true));
}

  bool
NodeManager::NodeIsExternal (Ptr<Node> node) const
{
  NS_LOG_FUNCTION;
  for (uint32_t i = 0; i < GetNNodes (); ++i)
    {
      if (node == m_nodes[i].first)
        {
          return m_nodes[i].second;
        }
    }
  NS_ASSERT_MSG (0, "NodeManager::NodeIsExternal(): Unknown node");
  return false;
}

  Ptr<Node>
NodeManager::CreateNode (void)
{
  NS_LOG_FUNCTION;
  return CreateObject<Node> ();
}

  uint32_t
NodeManager::GetNNodes (void) const
{
  NS_LOG_FUNCTION;
  return m_nodes.size ();
}

  Ptr<Node> 
NodeManager::GetNode (uint32_t index) const
{
  NS_LOG_FUNCTION;
  return m_nodes[index].first;
}

NS_OBJECT_ENSURE_REGISTERED (NetDeviceManager);

  TypeId 
NetDeviceManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("NetDeviceManager")
    .SetParent<Object> ()
    .AddConstructor<NetDeviceManager> ();

  return tid;
}

NetDeviceManager::NetDeviceManager ()
  : m_channel (0)
{
  NS_LOG_FUNCTION;
}

NetDeviceManager::~NetDeviceManager ()
{
  NS_LOG_FUNCTION;
}

  void
NetDeviceManager::Create (uint32_t n)
{
  NS_LOG_FUNCTION;
  PreCreate ();
  m_channel = CreateChannel ();

  for (uint32_t i = 0; i < n; ++i)
    {
      Ptr<NetDevice> device = CreateNetDevice ();
      m_devices.push_back (device);
    }
  PostCreate ();
}

  void
NetDeviceManager::PreCreate (void)
{
  NS_LOG_FUNCTION;
}

  void
NetDeviceManager::PostCreate (void)
{
  NS_LOG_FUNCTION;
}

  Ptr<NetDevice>
NetDeviceManager::CreateNetDevice (void)
{
  NS_LOG_FUNCTION;
  return 0;
}

  Ptr<Channel>
NetDeviceManager::CreateChannel (void)
{
  NS_LOG_FUNCTION;
  return 0;
}

  void
NetDeviceManager::AttachNetDevice (Ptr<NetDevice> device)
{
  NS_LOG_FUNCTION;
}

  void
NetDeviceManager::Install (Ptr<NodeManager> nodes)
{
  NS_LOG_FUNCTION;
  PreInstall ();
  uint32_t nNodes = nodes->GetNNodes();
  uint32_t nDevices = GetNDevices();

  NS_ASSERT_MSG (nNodes >= nDevices, "NetDeviceManager::Install(): "
    "More NetDevices than Nodes");
//
// What to do about external nodes.  An external node is one that has been
// previously initialized in some way and added to the node manager we were
// passed.  In this case, we are just being asked to install the particular
// device on that node, so we expect we can really ignore the external case.
// If you've already assigned a net device, you'll get another one.
//
  for (uint32_t i = 0; i < nNodes; ++i)
    {
      Ptr<Node> node = nodes->GetNode (i);
      Ptr<NetDevice> device = GetDevice (i);
      NS_LOG_LOGIC ("Adding device " << device << " to node " << node);
      device->SetNode (node);
      node->AddDevice (device);
    }
  PostInstall ();
}

  void
NetDeviceManager::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
NetDeviceManager::PostInstall (void)
{
  NS_LOG_FUNCTION;
}

  uint32_t 
NetDeviceManager::GetNDevices (void) const
{
  NS_LOG_FUNCTION;
  return m_devices.size ();
}

  Ptr<NetDevice> 
NetDeviceManager::GetDevice (uint32_t index) const
{
  NS_LOG_FUNCTION;
  return m_devices[index];
}

  Ptr<Channel> 
NetDeviceManager::GetChannel (void) const
{
  NS_LOG_FUNCTION;
  return m_channel;
}

NS_OBJECT_ENSURE_REGISTERED (ProtocolManager);

  TypeId 
ProtocolManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("ProtocolManager")
    .SetParent<Object> ()
    .AddConstructor<ProtocolManager> ();
  return tid;
}

ProtocolManager::ProtocolManager ()
{
  NS_LOG_FUNCTION;
}

ProtocolManager::~ProtocolManager ()
{
  NS_LOG_FUNCTION;
}

  void
ProtocolManager::Install (Ptr<NodeManager> nodes)
{
  NS_LOG_FUNCTION;
  PreInstall ();
  uint32_t nNodes = nodes->GetNNodes();
//
// What to do about external nodes.  An external node is one that has been
// previously initialized in some way and added to the node manager we were
// passed.  In this case, we could assume that *the* expected protocol suite
// has been previously added, but we won't make that assumption.  Perhaps the
// external node has another protocol suite and we then need to add another
// different suite.  So, we'll pass the external node flag on to the
// CreateProtocols method and let it figure out the right thing to do.
//
  for (uint32_t i = 0; i < nNodes; ++i)
    {
      Ptr<Node> node = nodes->GetNode (i);
      CreateProtocols (node, nodes->NodeIsExternal (node));
    }
  PostInstall ();
}

  void
ProtocolManager::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
ProtocolManager::PostInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
ProtocolManager::CreateProtocols (Ptr<Node> node, bool external)
{
  NS_LOG_FUNCTION;
}

NS_OBJECT_ENSURE_REGISTERED (AddressManager);

  TypeId 
AddressManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("AddressManager")
    .SetParent<Object> ()
    .AddConstructor<AddressManager> ();
  return tid;
}

AddressManager::AddressManager ()
{
  NS_LOG_FUNCTION;
}

AddressManager::~AddressManager ()
{
  NS_LOG_FUNCTION;
}

  void
AddressManager::Install (Ptr<NetDeviceManager> devices)
{
  NS_LOG_FUNCTION;
  PreInstall ();
  uint32_t nDevices = devices->GetNDevices();
//
// What to do about external nodes.  An external node is one that has been
// previously initialized in some way and added to a node manager.
// We are given a net device manager in this case.  The net device manager
// only holds virgin net devices we just created, so we can ignore the 
// external node case here.
//
  for (uint32_t i = 0; i < nDevices; ++i)
    {
      Ptr<NetDevice> device = devices->GetDevice (i);
      NS_ASSERT_MSG(device, "AddressManager::Install(): bad device");
      AssignAddress (device);
    }
  PostInstall ();
}

  void
AddressManager::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
AddressManager::PostInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
AddressManager::AssignAddress (Ptr<NetDevice> device)
{
  NS_LOG_FUNCTION;
}

NS_OBJECT_ENSURE_REGISTERED (ApplicationManager);

  TypeId 
ApplicationManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("ApplicationManager")
    .SetParent<Object> ()
    .AddConstructor<ApplicationManager> ();
  return tid;
}

ApplicationManager::ApplicationManager ()
{
  NS_LOG_FUNCTION;
}

ApplicationManager::~ApplicationManager ()
{
  NS_LOG_FUNCTION;
}

  void
ApplicationManager::Install (Ptr<NodeManager> nodes)
{
  NS_LOG_FUNCTION;
  PreInstall ();
  uint32_t nNodes = nodes->GetNNodes();
//
// What to do about external nodes.  An external node is one that has been
// previously initialized in some way and added to the node manager we were
// passed.  In this case, we could assume that the expected application suite
// has been previously added, but we won't make that assumption.  Perhaps the
// "other" application manager added a completely different set of 
// applications.  In this case, we need to add ours (or perhaps even some 
// subset of ours).  So, we'll pass the external node flag onto the
// CreateApplications method and let it figure out the right thing to do.
//
  for (uint32_t i = 0; i < nNodes; ++i)
    {
      Ptr<Node> node = nodes->GetNode (i);
      NS_ASSERT_MSG(node, "ApplicationManager::Install(): bad node");
      CreateApplications (node, nodes->NodeIsExternal (node));
    }
  PostInstall ();
}

  void
ApplicationManager::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
ApplicationManager::PostInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
ApplicationManager::CreateApplications (Ptr<Node> node, bool external)
{
  NS_LOG_FUNCTION;
}

NS_OBJECT_ENSURE_REGISTERED (NetworkManager);

  TypeId 
NetworkManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("NetworkManager")
    .SetParent<Object> ()
    .AddConstructor<NetworkManager> ();
  return tid;
}

NetworkManager::NetworkManager ()
  : m_nodeManager (0), 
    m_netdeviceManager (0), 
    m_protocolManager (0), 
    m_addressManager (0), 
    m_applicationManager (0)
{
  NS_LOG_FUNCTION;
  m_nodeManager = CreateObject<NodeManager> ();
  m_netdeviceManager = CreateObject<NetDeviceManager> ();
  m_protocolManager = CreateObject<ProtocolManager> ();
  m_addressManager = CreateObject<AddressManager> ();
  m_applicationManager = CreateObject<ApplicationManager> ();
}

NetworkManager::~NetworkManager ()
{
  NS_LOG_FUNCTION;

  m_nodeManager = 0;
  m_netdeviceManager = 0;
  m_protocolManager = 0;
  m_addressManager = 0;
  m_applicationManager = 0;
}

  void
NetworkManager::Create (uint32_t n)
{
  NS_LOG_FUNCTION;

  NS_ASSERT_MSG (m_nodeManager,
    "NetworkManager::Create (): No NodeManager set.");

  m_nodeManager->Create (n);
}

  void
NetworkManager::AddNode (Ptr<Node> node)
{
  NS_LOG_FUNCTION;
  m_nodeManager->Add (node);
}

  void
NetworkManager::Configure (void)
{
  NS_LOG_FUNCTION;

  NS_ASSERT_MSG (
    m_nodeManager &&
    m_netdeviceManager &&
    m_protocolManager &&
    m_addressManager &&
    m_applicationManager, 
    "NetworkManager::Create (): All managers must be set before calling.");

  PreConfigure ();
  ConfigureNetwork ();
  PostConfigure ();
}

  void
NetworkManager::PreConfigure (void)
{
  NS_LOG_FUNCTION;
}

  void
NetworkManager::PostConfigure (void)
{
  NS_LOG_FUNCTION;
}

  Ptr<NodeManager> 
NetworkManager::GetNodeManager (void)
{
  NS_LOG_FUNCTION;
  return m_nodeManager;
}

  void
NetworkManager::ConfigureNetwork (void)
{
  NS_LOG_FUNCTION;

  uint32_t n = m_nodeManager->GetNNodes ();

  m_netdeviceManager->Create (n);
  m_netdeviceManager->Install (m_nodeManager);
  m_protocolManager->Install (m_nodeManager);
  m_addressManager->Install (m_netdeviceManager);
  m_applicationManager->Install (m_nodeManager);
}

  void
NetworkManager::SetNodeManager (Ptr<NodeManager> nodeManager)
{
  NS_LOG_FUNCTION;
  m_nodeManager = nodeManager;
}

  Ptr<NetDeviceManager> 
NetworkManager::GetNetDeviceManager (void)
{
  NS_LOG_FUNCTION;
  return m_netdeviceManager;
}

  void
NetworkManager::SetNetDeviceManager (Ptr<NetDeviceManager> netdeviceManager)
{
  NS_LOG_FUNCTION;
  m_netdeviceManager = netdeviceManager;
}
 
  Ptr<ProtocolManager> 
NetworkManager::GetProtocolManager (void)
{
  NS_LOG_FUNCTION;
  return m_protocolManager;
}

  void 
NetworkManager::SetProtocolManager (Ptr<ProtocolManager> protocolManager)
{
  NS_LOG_FUNCTION;
  m_protocolManager = protocolManager;
}

  Ptr<AddressManager> 
NetworkManager::GetAddressManager (void)
{
  NS_LOG_FUNCTION;
  return m_addressManager;
}

  void 
NetworkManager::SetAddressManager (Ptr<AddressManager> addressManager)
{
  NS_LOG_FUNCTION;
  m_addressManager = addressManager;
}

  Ptr<ApplicationManager> 
NetworkManager::GetApplicationManager (void)
{
  NS_LOG_FUNCTION;
  return m_applicationManager;
}

  void 
NetworkManager::SetApplicationManager (Ptr<ApplicationManager> appManager)
{
  NS_LOG_FUNCTION;
  m_applicationManager = appManager;
}

} // namespace ns3
