/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TOPOLOGY_H__
#define __TOPOLOGY_H__

#include <vector>
#include "ns3/ptr.h"
#include "ns3/object.h"
#include "ns3/channel.h"
#include "ns3/net-device.h"
#include "ns3/node.h"

namespace ns3 {

/**
 * \ingroup Topology
 *
 * \section Topology Overview
 *
 */

class NodeManager : public Object
{
public:
  static TypeId GetTypeId (void);

  NodeManager ();
  virtual ~NodeManager ();

  void Create (uint32_t n);
  void Add (Ptr<Node> node);
  bool NodeIsExternal (Ptr<Node> node) const;

  uint32_t GetNNodes (void) const;
  Ptr<Node> GetNode (uint32_t index) const;

protected:
  virtual void PreCreate (void);
  virtual void PostCreate (void);
  virtual Ptr<Node> CreateNode (void);

private:
  typedef std::vector<std::pair<Ptr<Node>, bool> > VectorOfNodePair_t;
  VectorOfNodePair_t m_nodes;
};

class NetDeviceManager : public Object
{
public:
  static TypeId GetTypeId (void);

  NetDeviceManager ();
  virtual ~NetDeviceManager ();

  virtual void Create (uint32_t n);
  virtual void Install (Ptr<NodeManager> nodes);

  virtual uint32_t GetNDevices (void) const;
  virtual Ptr<NetDevice> GetDevice (uint32_t index) const;

  virtual Ptr<Channel> GetChannel (void) const;

  virtual void PreCreate (void);
  virtual void PostCreate (void);

  virtual void PreInstall (void);
  virtual void PostInstall (void);

  virtual Ptr<NetDevice> CreateNetDevice (void);
  virtual Ptr<Channel> CreateChannel (void);
  virtual void AttachNetDevice (Ptr<NetDevice> device);

  Ptr<Channel> m_channel;

  void Connect (Ptr<NetDevice> netDevice);

  typedef std::vector<Ptr<NetDevice> > VectorOfNetDevice_t;
  VectorOfNetDevice_t m_devices;
};

class ProtocolManager : public Object
{
public:
  static TypeId GetTypeId (void);

  ProtocolManager ();
  virtual ~ProtocolManager ();

  void Install (Ptr<NodeManager> nodes);

  virtual void PreInstall (void);
  virtual void PostInstall (void);

  virtual void CreateProtocols (Ptr<Node> node, bool external);
};

class AddressManager : public Object
{
public:
  static TypeId GetTypeId (void);

  AddressManager ();
  virtual ~AddressManager ();

  void Install (Ptr<NetDeviceManager> devices);

  virtual void PreInstall (void);
  virtual void PostInstall (void);

  virtual void AssignAddress (Ptr<NetDevice> device);
};

class ApplicationManager : public Object
{
public:
  static TypeId GetTypeId (void);

  ApplicationManager ();
  virtual ~ApplicationManager ();

  void Install (Ptr<NodeManager> nodes);

  virtual void PreInstall (void);
  virtual void PostInstall (void);

  virtual void CreateApplications (Ptr<Node> node, bool external);
};

class NetworkManager : public Object
{
public:
  static TypeId GetTypeId (void);

  NetworkManager ();
  virtual ~NetworkManager ();

  void Create (uint32_t n);
  void AddNode (Ptr<Node> node);
  void Configure (void);

  Ptr<NodeManager> GetNodeManager (void);
  void SetNodeManager (Ptr<NodeManager> nodeManager);

  Ptr<NetDeviceManager> GetNetDeviceManager (void);
  void SetNetDeviceManager (Ptr<NetDeviceManager> netdeviceManager);
 
  Ptr<ProtocolManager> GetProtocolManager (void);
  void SetProtocolManager (Ptr<ProtocolManager> protocolManager);

  Ptr<AddressManager> GetAddressManager (void);
  void SetAddressManager (Ptr<AddressManager> addressManager);

  Ptr<ApplicationManager> GetApplicationManager (void);
  void SetApplicationManager (Ptr<ApplicationManager> applicationManager);

protected:
  virtual void PreConfigure (void);
  virtual void PostConfigure (void);
  virtual void ConfigureNetwork (void);

private:
  Ptr<NodeManager> m_nodeManager;
  Ptr<NetDeviceManager> m_netdeviceManager;
  Ptr<ProtocolManager> m_protocolManager;
  Ptr<AddressManager> m_addressManager;
  Ptr<ApplicationManager> m_applicationManager;
};

} // namespace ns3

#endif //__TOPOLOGY_H__
